"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = dsqlMethodCrud;
const lodash_1 = __importDefault(require("lodash"));
const deserialize_query_1 = __importDefault(require("../deserialize-query"));
const ejson_1 = __importDefault(require("../ejson"));
const numberfy_1 = __importDefault(require("../numberfy"));
const crud_1 = __importDefault(require("./crud"));
function dsqlMethodCrud(_a) {
    return __awaiter(this, arguments, void 0, function* ({ method, tableName, addUser, user, extraData, transform, existingData, body, query, targetId, sanitize, }) {
        let result = {
            success: false,
        };
        try {
            let finalBody = body;
            let finalQuery = (0, deserialize_query_1.default)(query);
            let LIMIT = 10;
            let PAGE = 1;
            let OFFSET = (PAGE - 1) * LIMIT;
            if (finalQuery) {
                const newFinalQuery = lodash_1.default.cloneDeep(finalQuery);
                Object.keys(newFinalQuery).forEach((key) => {
                    const value = newFinalQuery[key];
                    if (typeof value == "string" && value.match(/^\{|^\[/)) {
                        newFinalQuery[key] = ejson_1.default.stringify(value);
                    }
                    if (value == "true") {
                        newFinalQuery[key] = true;
                    }
                    if (value == "false") {
                        newFinalQuery[key] = false;
                    }
                });
                if (newFinalQuery.limit)
                    LIMIT = (0, numberfy_1.default)(newFinalQuery.limit);
                if (newFinalQuery.page)
                    PAGE = (0, numberfy_1.default)(newFinalQuery.page);
                OFFSET = (PAGE - 1) * LIMIT;
                finalQuery = newFinalQuery;
            }
            else {
                finalQuery = {};
            }
            let finalData = finalBody
                ? Object.assign(Object.assign({}, finalBody), extraData)
                : {};
            if ((user === null || user === void 0 ? void 0 : user.id) && addUser) {
                finalData = Object.assign(Object.assign({}, finalData), { [addUser.field]: String(user.id) });
            }
            if (transform) {
                finalData = (yield transform({
                    data: finalData,
                    existingData: existingData,
                    user,
                    reqMethod: method,
                }));
                finalQuery = yield transform({
                    query: finalQuery,
                    user,
                    reqMethod: method,
                });
            }
            switch (method) {
                case "GET":
                    const GET_RESULT = yield (0, crud_1.default)({
                        action: "get",
                        table: tableName,
                        query: finalQuery
                            ? Object.assign(Object.assign({}, finalQuery), { query: Object.assign(Object.assign({}, finalQuery.query), ((user === null || user === void 0 ? void 0 : user.id) && addUser
                                    ? {
                                        [addUser.field]: {
                                            value: String(user.id),
                                        },
                                    }
                                    : undefined)), limit: LIMIT, offset: OFFSET })
                            : undefined,
                        sanitize,
                    });
                    result = {
                        success: Boolean(GET_RESULT === null || GET_RESULT === void 0 ? void 0 : GET_RESULT.success),
                        payload: GET_RESULT === null || GET_RESULT === void 0 ? void 0 : GET_RESULT.payload,
                        msg: GET_RESULT === null || GET_RESULT === void 0 ? void 0 : GET_RESULT.msg,
                        error: GET_RESULT === null || GET_RESULT === void 0 ? void 0 : GET_RESULT.error,
                        queryObject: GET_RESULT === null || GET_RESULT === void 0 ? void 0 : GET_RESULT.queryObject,
                    };
                    break;
                case "POST":
                    const POST_RESULT = yield (0, crud_1.default)({
                        action: "insert",
                        table: tableName,
                        data: finalData,
                        sanitize,
                    });
                    result = {
                        success: Boolean(POST_RESULT === null || POST_RESULT === void 0 ? void 0 : POST_RESULT.success),
                        payload: POST_RESULT === null || POST_RESULT === void 0 ? void 0 : POST_RESULT.payload,
                        msg: POST_RESULT === null || POST_RESULT === void 0 ? void 0 : POST_RESULT.msg,
                        error: POST_RESULT === null || POST_RESULT === void 0 ? void 0 : POST_RESULT.error,
                    };
                    break;
                case "PUT":
                    const PUT_RESULT = yield (0, crud_1.default)({
                        action: "update",
                        table: tableName,
                        data: finalData,
                        targetId,
                        sanitize,
                    });
                    result = {
                        success: Boolean(PUT_RESULT === null || PUT_RESULT === void 0 ? void 0 : PUT_RESULT.success),
                        payload: PUT_RESULT === null || PUT_RESULT === void 0 ? void 0 : PUT_RESULT.payload,
                        msg: PUT_RESULT === null || PUT_RESULT === void 0 ? void 0 : PUT_RESULT.msg,
                        error: PUT_RESULT === null || PUT_RESULT === void 0 ? void 0 : PUT_RESULT.error,
                    };
                    break;
                case "DELETE":
                    const DELETE_RESULT = yield (0, crud_1.default)({
                        action: "delete",
                        table: tableName,
                        targetId,
                        sanitize,
                    });
                    result = {
                        success: Boolean(DELETE_RESULT === null || DELETE_RESULT === void 0 ? void 0 : DELETE_RESULT.success),
                        payload: DELETE_RESULT === null || DELETE_RESULT === void 0 ? void 0 : DELETE_RESULT.payload,
                        msg: DELETE_RESULT === null || DELETE_RESULT === void 0 ? void 0 : DELETE_RESULT.msg,
                        error: DELETE_RESULT === null || DELETE_RESULT === void 0 ? void 0 : DELETE_RESULT.error,
                    };
                    break;
                default:
                    break;
            }
            return result;
        }
        catch (error) {
            return result;
        }
    });
}
